@@ -119,6 +119,10 @@ def load_modules(basedir, suffix='.py'):
     for modname, sourcefile in list_dir('', basedir):
         if modname == 'runtests':
             continue
+        if modname == 'test_asyncio' and sys.version_info <= (3, 3):
+            print("Skipping '{0}': need at least Python 3.3".format(modname),
+                  file=sys.stderr)
+            continue
         try:
             mod = load_module(modname, sourcefile)
             mods.append((mod, sourcefile))
@@ -261,20 +265,27 @@ def runtests():
                                 )
         cov.start()
 
-    finder = TestsFinder(args.testsdir, includes, excludes)
     logger = logging.getLogger()
     if v == 0:
-        logger.setLevel(logging.CRITICAL)
+        level = logging.CRITICAL
     elif v == 1:
-        logger.setLevel(logging.ERROR)
+        level = logging.ERROR
     elif v == 2:
-        logger.setLevel(logging.WARNING)
+        level = logging.WARNING
     elif v == 3:
-        logger.setLevel(logging.INFO)
+        level = logging.INFO
     elif v >= 4:
-        logger.setLevel(logging.DEBUG)
+        level = logging.DEBUG
+    logging.basicConfig(level=level)
+
+    finder = TestsFinder(args.testsdir, includes, excludes)
     if catchbreak:
         installHandler()
+    import trollius.coroutines
+    if trollius.coroutines._DEBUG:
+        print("Run tests in debug mode")
+    else:
+        print("Run tests in release mode")
     try:
         if args.forever:
             while True:
