@@ -1,5 +1,6 @@
 """Tests for tasks.py."""
 
+import os
 import re
 import sys
 import types
@@ -134,6 +135,8 @@ class TaskTests(test_utils.TestCase):
             asyncio.async('ok')
 
     def test_task_repr(self):
+        self.loop.set_debug(False)
+
         @asyncio.coroutine
         def noop():
             yield From(None)
@@ -196,6 +199,8 @@ class TaskTests(test_utils.TestCase):
                          "<Task finished %s result='abc'>" % coro)
 
     def test_task_repr_coro_decorator(self):
+        self.loop.set_debug(False)
+
         @asyncio.coroutine
         def notmuch():
             # notmuch() function doesn't use yield: it will be wrapped by
@@ -262,6 +267,8 @@ class TaskTests(test_utils.TestCase):
         self.loop.run_until_complete(t)
 
     def test_task_repr_wait_for(self):
+        self.loop.set_debug(False)
+
         @asyncio.coroutine
         def wait_for(fut):
             res = yield From(fut)
@@ -572,6 +579,21 @@ class TaskTests(test_utils.TestCase):
         self.assertTrue(fut.done())
         self.assertTrue(fut.cancelled())
 
+    def test_wait_for_race_condition(self):
+
+        def gen():
+            yield 0.1
+            yield 0.1
+            yield 0.1
+
+        loop = self.new_test_loop(gen)
+
+        fut = asyncio.Future(loop=loop)
+        task = asyncio.wait_for(fut, timeout=0.2, loop=loop)
+        loop.call_later(0.1, fut.set_result, "ok")
+        res = loop.run_until_complete(task)
+        self.assertEqual(res, "ok")
+
     def test_wait(self):
 
         def gen():
@@ -1564,6 +1586,7 @@ class TaskTests(test_utils.TestCase):
             try:
                 cw.send(arg)
             except StopIteration as ex:
+                ex.raised = True
                 return ex.value
             else:
                 raise AssertionError('StopIteration was expected')
@@ -1660,6 +1683,20 @@ class TaskTests(test_utils.TestCase):
         self.check_soure_traceback(task._source_traceback, -1)
         self.loop.run_until_complete(task)
 
+    def test_coroutine_class(self):
+        # Trollius issue #9
+        self.loop.set_debug(True)
+
+        class MyClass(object):
+            def __call__(self):
+                return 7
+
+        obj = MyClass()
+        coro_func = asyncio.coroutine(obj)
+        coro_obj = coro_func()
+        res = self.loop.run_until_complete(coro_obj)
+        self.assertEqual(res, 7)
+
 
 class GatherTestsBase:
 
@@ -1732,16 +1769,20 @@ class GatherTestsBase:
         self.assertEqual(fut.result(), [3, 1, exc, exc2])
 
     def test_env_var_debug(self):
+        aio_path = os.path.dirname(os.path.dirname(asyncio.__file__))
+
         code = '\n'.join((
             'import trollius.coroutines',
             'print(trollius.coroutines._DEBUG)'))
 
         sts, stdout, stderr = assert_python_ok('-c', code,
-                                               TROLLIUSDEBUG='')
+                                               TROLLIUSDEBUG='',
+                                               PYTHONPATH=aio_path)
         self.assertEqual(stdout.rstrip(), b'False')
 
         sts, stdout, stderr = assert_python_ok('-c', code,
-                                               TROLLIUSDEBUG='1')
+                                               TROLLIUSDEBUG='1',
+                                               PYTHONPATH=aio_path)
         self.assertEqual(stdout.rstrip(), b'True')
 
 
