@@ -16,10 +16,6 @@ try:
 except ImportError:
     # Python 2.6
     from .py27_weakrefset import WeakSet
-try:
-    import asyncio
-except ImportError:
-    asyncio = None
 
 from . import compat
 from . import coroutines
@@ -30,12 +26,6 @@ from .locks import Lock, Condition, Semaphore, _ContextManager
 from .coroutines import coroutine, From, Return, iscoroutinefunction, iscoroutine
 
 
-if asyncio is not None:
-    # Accept also asyncio Future objects for interoperability
-    _FUTURE_CLASSES = (futures.Future, asyncio.Future)
-else:
-    _FUTURE_CLASSES = futures.Future
-
 _PY34 = (sys.version_info >= (3, 4))
 
 
@@ -100,9 +90,9 @@ class Task(futures.Future):
         # status is still pending
         self._log_destroy_pending = True
 
-    # On Python 3.3 or older, objects with a destructor part of a reference
-    # cycle are never destroyed. It's not more the case on Python 3.4 thanks to
-    # the PEP 442.
+    # On Python 3.3 or older, objects with a destructor that are part of a
+    # reference cycle are never destroyed. That's not the case any more on
+    # Python 3.4 thanks to the PEP 442.
     if _PY34:
         def __del__(self):
             if self._state == futures._PENDING and self._log_destroy_pending:
@@ -115,30 +105,19 @@ class Task(futures.Future):
                 self._loop.call_exception_handler(context)
             futures.Future.__del__(self)
 
-    def __repr__(self):
-        info = []
+    def _repr_info(self):
+        info = super(Task, self)._repr_info()
+
         if self._must_cancel:
-            info.append('cancelling')
-        else:
-            info.append(self._state.lower())
+            # replace status
+            info[0] = 'cancelling'
 
         coro = coroutines._format_coroutine(self._coro)
-        info.append('coro=<%s>' % coro)
-
-        if self._source_traceback:
-            frame = self._source_traceback[-1]
-            info.append('created at %s:%s' % (frame[0], frame[1]))
-
-        if self._state == futures._FINISHED:
-            info.append(self._format_result())
-
-        if self._callbacks:
-            info.append(self._format_callbacks())
+        info.insert(1, 'coro=<%s>' % coro)
 
         if self._fut_waiter is not None:
-            info.append('wait_for=%r' % self._fut_waiter)
-
-        return '<%s %s>' % (self.__class__.__name__, ' '.join(info))
+            info.insert(2, 'wait_for=%r' % self._fut_waiter)
+        return info
 
     def get_stack(self, limit=None):
         """Return the list of stack frames for this task's coroutine.
@@ -189,7 +168,8 @@ class Task(futures.Future):
         This produces output similar to that of the traceback module,
         for the frames retrieved by get_stack().  The limit argument
         is passed to get_stack().  The file argument is an I/O stream
-        to which the output goes; by default it goes to sys.stderr.
+        to which the output is written; by default output is written
+        to sys.stderr.
         """
         extracted_list = []
         checked = set()
@@ -218,18 +198,18 @@ class Task(futures.Future):
                 print(line, file=file, end='')
 
     def cancel(self):
-        """Request this task to cancel itself.
+        """Request that this task cancel itself.
 
         This arranges for a CancelledError to be thrown into the
         wrapped coroutine on the next cycle through the event loop.
         The coroutine then has a chance to clean up or even deny
         the request using try/except/finally.
 
-        Contrary to Future.cancel(), this does not guarantee that the
+        Unlike Future.cancel, this does not guarantee that the
         task will be cancelled: the exception might be caught and
-        acted upon, delaying cancellation of the task or preventing it
-        completely.  The task may also return a value or raise a
-        different exception.
+        acted upon, delaying cancellation of the task or preventing
+        cancellation completely.  The task may also return a value or
+        raise a different exception.
 
         Immediately after this method is called, Task.cancelled() will
         not return True (unless the task was already cancelled).  A
@@ -288,15 +268,20 @@ class Task(futures.Future):
             raise
         else:
             if coroutines._DEBUG:
-                if not isinstance(result, coroutines.FromWrapper):
-                    self._loop.call_soon(
-                        self._step, None,
-                        RuntimeError("yield used without From"))
-                    return
-                result = result.obj
-            else:
-                if isinstance(result, coroutines.FromWrapper):
+                if not coroutines._coroutine_at_yield_from(self._coro):
+                    # trollius coroutine must "yield From(...)"
+                    if not isinstance(result, coroutines.FromWrapper):
+                        self._loop.call_soon(
+                            self._step, None,
+                            RuntimeError("yield used without From"))
+                        return
                     result = result.obj
+                else:
+                    # asyncio coroutine using "yield from ..."
+                    if isinstance(result, coroutines.FromWrapper):
+                        result = result.obj
+            elif isinstance(result, coroutines.FromWrapper):
+                result = result.obj
 
             if iscoroutine(result):
                 result = async(result, loop=self._loop)
@@ -305,7 +290,7 @@ class Task(futures.Future):
                 coro = _lock_coroutine(result)
                 result = self._loop.create_task(coro)
 
-            if isinstance(result, _FUTURE_CLASSES):
+            if isinstance(result, futures._FUTURE_CLASSES):
                 # Yielded Future must come from Future.__iter__().
                 result.add_done_callback(self._wakeup)
                 self._fut_waiter = result
@@ -361,7 +346,7 @@ def wait(fs, loop=None, timeout=None, return_when=ALL_COMPLETED):
     Note: This does not raise TimeoutError! Futures that aren't done
     when the timeout occurs are returned in the second set.
     """
-    if isinstance(fs, futures.Future) or coroutines.iscoroutine(fs):
+    if isinstance(fs, futures._FUTURE_CLASSES) or coroutines.iscoroutine(fs):
         raise TypeError("expect a list of futures, not %s" % type(fs).__name__)
     if not fs:
         raise ValueError('Set of coroutines/Futures is empty.')
@@ -377,9 +362,9 @@ def wait(fs, loop=None, timeout=None, return_when=ALL_COMPLETED):
     raise Return(result)
 
 
-def _release_waiter(waiter, value=True, *args):
+def _release_waiter(waiter, *args):
     if not waiter.done():
-        waiter.set_result(value)
+        waiter.set_result(None)
 
 
 @coroutine
@@ -404,14 +389,17 @@ def wait_for(fut, timeout, loop=None):
         raise Return((yield From(fut)))
 
     waiter = futures.Future(loop=loop)
-    timeout_handle = loop.call_later(timeout, _release_waiter, waiter, False)
-    cb = functools.partial(_release_waiter, waiter, True)
+    timeout_handle = loop.call_later(timeout, _release_waiter, waiter)
+    cb = functools.partial(_release_waiter, waiter)
 
     fut = async(fut, loop=loop)
     fut.add_done_callback(cb)
 
     try:
-        if (yield From(waiter)):
+        # wait until the future completes or the timeout
+        yield From(waiter)
+
+        if fut.done():
             raise Return(fut.result())
         else:
             fut.remove_done_callback(cb)
@@ -443,7 +431,7 @@ def _wait(fs, timeout, return_when, loop):
             if timeout_handle is not None:
                 timeout_handle.cancel()
             if not waiter.done():
-                waiter.set_result(False)
+                waiter.set_result(None)
 
     for f in fs:
         f.add_done_callback(_on_completion)
@@ -483,7 +471,7 @@ def as_completed(fs, loop=None, timeout=None):
 
     Note: The futures 'f' are not necessarily members of fs.
     """
-    if isinstance(fs, futures.Future) or coroutines.iscoroutine(fs):
+    if isinstance(fs, futures._FUTURE_CLASSES) or coroutines.iscoroutine(fs):
         raise TypeError("expect a list of futures, not %s" % type(fs).__name__)
     loop = loop if loop is not None else events.get_event_loop()
     todo = set(async(f, loop=loop) for f in set(fs))
@@ -518,7 +506,7 @@ def as_completed(fs, loop=None, timeout=None):
     if todo and timeout is not None:
         timeout_handle = loop.call_later(timeout, _on_timeout)
     for _ in range(len(todo)):
-        yield From(_wait_for_one())
+        yield _wait_for_one()
 
 
 @coroutine
@@ -542,7 +530,7 @@ def async(coro_or_future, loop=None):
     # FIXME: only check if coroutines._DEBUG is True?
     if isinstance(coro_or_future, coroutines.FromWrapper):
         coro_or_future = coro_or_future.obj
-    if isinstance(coro_or_future, futures.Future):
+    if isinstance(coro_or_future, futures._FUTURE_CLASSES):
         if loop is not None and loop is not coro_or_future._loop:
             raise ValueError('loop argument must agree with Future')
         return coro_or_future
@@ -609,7 +597,7 @@ def gather(*coros_or_futures, **kw):
 
     arg_to_fut = {}
     for arg in set(coros_or_futures):
-        if not isinstance(arg, futures.Future):
+        if not isinstance(arg, futures._FUTURE_CLASSES):
             fut = async(arg, loop=loop)
             if loop is None:
                 loop = fut._loop
