@@ -3,6 +3,7 @@
 import collections
 import contextlib
 import io
+import logging
 import os
 import re
 import socket
@@ -39,6 +40,8 @@ from . import events
 from . import futures
 from . import selectors
 from . import tasks
+from .coroutines import coroutine
+from .log import logger
 
 
 if sys.platform == 'win32':  # pragma: no cover
@@ -143,12 +146,15 @@ def dummy_ssl_context():
 
 
 def run_briefly(loop, steps=1):
-    @tasks.coroutine
+    @coroutine
     def once():
         pass
     for step in range(steps):
         gen = once()
-        t = tasks.Task(gen, loop=loop)
+        t = loop.create_task(gen)
+        # Don't log a warning if the task is not done after run_until_complete().
+        # It occurs if the loop is stopped or if a task raises a BaseException.
+        t._log_destroy_pending = False
         try:
             loop.run_until_complete(t)
         finally:
@@ -532,3 +538,33 @@ class TestCase(_TestCase):
                 msg = '%s: %r not found in %r' % (msg, expected_regex.pattern, text)
                 raise self.failureException(msg)
 
+    def check_soure_traceback(self, source_traceback, lineno_delta):
+        frame = sys._getframe(1)
+        filename = frame.f_code.co_filename
+        lineno = frame.f_lineno + lineno_delta
+        name = frame.f_code.co_name
+        self.assertIsInstance(source_traceback, list)
+        self.assertEqual(source_traceback[-1][:3],
+                         (filename,
+                          lineno,
+                          name))
+
+
+@contextlib.contextmanager
+def disable_logger():
+    """Context manager to disable asyncio logger.
+
+    For example, it can be used to ignore warnings in debug mode.
+    """
+    old_level = logger.level
+    try:
+        logger.setLevel(logging.CRITICAL+1)
+        yield
+    finally:
+        logger.setLevel(old_level)
+
+def mock_nonblocking_socket():
+    """Create a mock of a non-blocking socket."""
+    sock = mock.Mock(socket.socket)
+    sock.gettimeout.return_value = 0.0
+    return sock
